@@ -12,8 +12,8 @@ const platforms = sauceBrowsers([
   { name: 'firefox', version: ['oldest', 'latest'] },
   { name: 'internet explorer', version: 'oldest..latest' },
   { name: 'iphone', version: ['oldest', 'latest'] },
-  { name: 'safari', version: 'oldest..latest' },
-  { name: 'microsoftedge', version: 'oldest..latest' }
+  { name: 'safari', version: ['oldest', 'latest'] },
+  { name: 'microsoftedge', version: ['oldest', 'latest'] }
 ]).then((platforms) => {
   return platforms.map((platform) => {
     const ret = {
@@ -29,12 +29,12 @@ const platforms = sauceBrowsers([
 });
 
 run(path.join(__dirname, 'test.js'), 'saucelabs', {
+  jobInfo: { name: pkg.name, build: process.env.GITHUB_RUN_ID },
   html: path.join(__dirname, 'index.html'),
   accessKey: process.env.SAUCE_ACCESS_KEY,
   username: process.env.SAUCE_USERNAME,
   browserify: true,
   disableSSL: true,
-  name: pkg.name,
   parallel: 5,
   platforms
 }).done((results) => {
